package models

import (
	"eta/eta_index_lib/models/mgo"
	"eta/eta_index_lib/utils"
	"fmt"
	"go.mongodb.org/mongo-driver/bson"
	"time"

	"github.com/beego/beego/v2/client/orm"
)

// BaseFromBusinessIndex
// @Description: 外部指标（商家系统）表
type BaseFromBusinessIndex struct {
	BaseFromBusinessIndexId int64     `orm:"column(base_from_business_index_id);pk"`
	IndexCode               string    `description:"指标编码"`
	IndexName               string    `description:"指标名称"`
	Unit                    string    `description:"单位"`
	Frequency               string    `description:"频度"`
	Source                  int       `description:"数据来源"`
	SourceName              string    `description:"数据来源名称"`
	StartDate               time.Time `description:"开始日期"`
	EndDate                 time.Time `description:"结束日期"`
	Remark                  string    `description:"备注字段"`
	BaseModifyTime          time.Time `description:"基础信息(名称，单位，频度)变更时间"`
	DataUpdateTime          time.Time `description:"最近一次数据发生变化的时间"`
	CreateTime              time.Time `description:"创建时间"`
	ModifyTime              time.Time `description:"修改时间"`
}

// EdbBusinessSource
// @Description: 自有数据（商家）指标来源
type EdbBusinessSource struct {
	EdbBusinessSourceId int64     `orm:"column(edb_business_source_id);pk"`
	SourceName          string    `description:"来源名称"` // 来源名称
	CreateTime          time.Time `description:"创建时间"` // 创建时间
}

// AddBusinessIndexReq
// @Description:  添加外部指标（商家）请求
type AddBusinessIndexReq struct {
	IndexCode  string               `description:"指标编码"`
	IndexName  string               `description:"指标名称"`
	Unit       string               `description:"单位"`
	Frequency  string               `description:"频度"`
	SourceName string               `description:"数据来源名称"`
	Remark     string               `description:"备注字段"`
	DataList   []AddBusinessDataReq `description:"指标数据"`
}

// AddBusinessDataReq
// @Description: 外部指标（商家系统）数据
type AddBusinessDataReq struct {
	Value float64 `description:"值"`
	Date  string  `description:"日期"`
}

func (m *BaseFromBusinessIndex) GetIndexItem(indexCode string) (item *BaseFromBusinessIndex, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM base_from_business_index WHERE index_code = ? `
	err = o.Raw(sql, indexCode).QueryRow(&item)
	return
}

func (m *BaseFromBusinessIndex) GetIndexCreate(terminalCode string) (items []*BaseFromBusinessIndex, err error) {
	o := orm.NewOrm()
	endTime := time.Now().Add(-2 * time.Minute).Format(utils.FormatDateTime)
	sql := `SELECT * FROM base_from_business_index WHERE index_name = '' AND create_time <= ? AND terminal_code = ? `
	_, err = o.Raw(sql, endTime, terminalCode).QueryRows(&items)
	return
}

// Add 新增
func (m *BaseFromBusinessIndex) Add() (err error) {
	o := orm.NewOrm()
	lastId, err := o.Insert(m)
	if err != nil {
		return
	}
	m.BaseFromBusinessIndexId = lastId

	return
}

func (m *BaseFromBusinessIndex) Update(cols []string) (err error) {
	o := orm.NewOrm()
	_, err = o.Update(m, cols...)
	return
}

func (m *BaseFromBusinessIndex) UpdateIndex(item *BaseFromBusinessIndex, updateCols []string) (err error) {
	if item == nil {
		return
	}
	if len(updateCols) == 0 {
		return
	}
	o := orm.NewOrm()
	_, err = o.Update(item, updateCols...)
	return
}

// GetEdbBusinessSourceItem
// @Description: 根据来源名称获取来源信息
// @author: Roc
// @receiver m
// @datetime 2024-04-25 18:09:03
// @param sourceName string
// @return item *EdbBusinessSource
// @return err error
func (m *EdbBusinessSource) GetEdbBusinessSourceItem(sourceName string) (item *EdbBusinessSource, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM edb_business_source WHERE source_name = ? `
	err = o.Raw(sql, sourceName).QueryRow(&item)
	return
}

// Add 新增
func (m *EdbBusinessSource) Add() (err error) {
	o := orm.NewOrm()
	lastId, err := o.Insert(m)
	if err != nil {
		return
	}
	m.EdbBusinessSourceId = lastId

	return
}

// GetEdbInfoMaxAndMinInfo 获取指标的最新数据记录信息
func (m BaseFromBusinessIndex) GetEdbInfoMaxAndMinInfo(edbCode string) (item *EdbInfoMaxAndMinInfo, err error) {
	mogDataObj := new(mgo.BaseFromBusinessData)
	pipeline := []bson.M{
		{"$match": bson.M{"index_code": edbCode}},
		{"$group": bson.M{
			"_id":       nil,
			"min_date":  bson.M{"$min": "$data_time"},
			"max_date":  bson.M{"$max": "$data_time"},
			"min_value": bson.M{"$min": "$value"},
			"max_value": bson.M{"$max": "$value"},
		}},
		{"$project": bson.M{"_id": 0}}, // 可选，如果不需要_id字段
	}
	result, err := mogDataObj.GetEdbInfoMaxAndMinInfo(pipeline)
	if err != nil {
		fmt.Println("BaseFromBusinessIndex GetEdbInfoMaxAndMinInfo Err:" + err.Error())
		return
	}

	if !result.MaxDate.IsZero() {
		whereQuery := bson.M{"index_code": edbCode, "data_time": result.MaxDate}
		selectParam := bson.D{{"value", 1}, {"_id", 0}}
		latestValue, tmpErr := mogDataObj.GetLatestValue(whereQuery, selectParam)
		if tmpErr != nil {
			err = tmpErr
			return
		}
		result.LatestValue = latestValue.Value
		result.EndValue = latestValue.Value
	}

	item = &EdbInfoMaxAndMinInfo{
		MinDate:     result.MinDate.Format(utils.FormatDate),
		MaxDate:     result.MaxDate.Format(utils.FormatDate),
		MinValue:    result.MinValue,
		MaxValue:    result.MaxValue,
		LatestValue: result.LatestValue,
		LatestDate:  result.LatestDate.Format(utils.FormatDate),
		EndValue:    result.EndValue,
	}

	return
}

// ModifyIndexMaxAndMinInfo
// @Description: 修改最大值和最小值信息
// @author: Roc
// @receiver m
// @datetime 2024-05-06 14:07:46
// @param indexCode string
// @param item *EdbInfoMaxAndMinInfo
// @param isIndexUpdateOrAdd bool
// @return err error
func (m *BaseFromBusinessIndex) ModifyIndexMaxAndMinInfo(indexCode string, item *EdbInfoMaxAndMinInfo, isIndexUpdateOrAdd bool) (err error) {
	o := orm.NewOrm()
	sql := ` UPDATE base_from_business_index SET start_date=?,end_date=?,modify_time=NOW() `
	if isIndexUpdateOrAdd {
		sql += `,data_update_time=NOW() `
	}
	sql += ` WHERE index_code=?`
	_, err = o.Raw(sql, item.MinDate, item.MaxDate, indexCode).Exec()
	return
}
